angular.module('utils', [])

  .factory('$localstorage', ['$window', function ($window) {
    return {
      set: function (key, value) {
        $window.localStorage[key] = value;
      },
      get: function (key, defaultValue) {
        return $window.localStorage[key] || defaultValue;
      },
      setObject: function (key, value) {
        $window.localStorage[key] = angular.toJson(value);
      },
      getObject: function (key) {
        return JSON.parse($window.localStorage[key] || '[]');
      },
      destroy: function (key) {
        $window.localStorage.removeItem(key);
      },
      log: function (key, defaultValue) {
        console.log($window.localStorage[key] || defaultValue);
      },
      logObject: function (key) {
        console.log(JSON.stringify($window.localStorage[key] || '{}'));
      }
    };
  }])

  .factory('$database', ['$cordovaSQLite', function ($cordovaSQLite) {
    var self = this;
    self.db = null;
    self.dbp = null;
    var query;

    //this updateTable function is for ilustrative use here, it can be changed to manage all queries indeed.
    updateTable = function (table, column, value, condition) {
      if (!condition) {
        query = "UPDATE " + table + " SET " + column + " = " + value;
      } else {
        query = "UPDATE " + table + " SET " + column + " = " + value + " WHERE " + condition;
      }
      console.log(query);
      return $cordovaSQLite.execute(self.db, query, []).then(function (result) {

        query = null;
        return result;
      }, function (error) {
        console.error(error);
        query = null;
        return error;
      });
    };

    return {
      //Added dbname as parameter to initDB function @2016-02-18
      initDB: function (dbname) {
        if (window.cordova) {
          //update data.db location, now without icloud/itunes visibility
          self.db = sqlitePlugin.openDatabase({name: dbname, location: 2, createFromLocation: 1});
          //we open a second database: qprogress added on @2016-06-15
          self.dbp = sqlitePlugin.openDatabase({name: 'qprogress', location: 2, createFromLocation: 1});
        } else {
          self.db = window.openDatabase(dbname, '1', 'my', 800 * 1024); // only available when WebSQL is available in Browser
          self.dbp = window.openDatabase('qprogress', '1', 'my', 800 * 1024); // only available when WebSQL is available in Browser
        }
        console.log('Databases opened: ' + dbname + ' & qprogress');
        //create table structure for progress
        query = "CREATE TABLE if not exists 'Progress' ('UserName' TEXT NOT NULL, 'ExamId' INTEGER NOT NULL, 'ExamTitle' TEXT NOT NULL, 'Date' INTEGER NOT NULL, 'Scored' INTEGER NOT NULL, 'Time' INTEGER NOT NULL);";
        return $cordovaSQLite.execute(self.dbp, query, []).then(function (result) {
          console.log("Creating Progress table");
          query = null;
          return result;
        }, function (error) {
          //this error tells that structure is already created, which is fine.
          console.error("Table Progress already exist");
          return error;
        });
        //return self.db; removed as no need it indeed.
      },
      closeDB: function () {
        self.db.close(function (success) {
          console.log('Main Database closed');
        });
        self.dbp.close(function (success) {
          console.log('Progress Database closed');
        });
      },
      unlockAllQuestions: function () {
        return updateTable('ExamType', 'Enabled', 1).//change update table
        then(function (result) {
          console.log("Unlock all questions");
          query = null;
          return result;
        }, function (error) {
          console.error(error);
          return error;
        });
      },
      setQuestionAsCorrect: function (id) {
        return updateTable('Question', 'Status', 2, 'Id=' + id).then(function (result) {
          console.log('Question ' + id + ' marked as correct');
          query = null;
          return result;
        }, function (error) {
          console.error(error);
          return error;
        });
      },
      setQuestionAsIncorrect: function (id) {
        return updateTable('Question', 'Status', 2, 'Id=' + id).then(function (result) {
          console.log('Question ' + id + ' marked as incorrect');
          query = null;
          return result;
        }, function (error) {
          console.error(error);
          return error;
        });
      },
      resetStatusOnQuestions: function () {
        return updateTable('Question', 'Status', 0).then(function (result) {
          console.log("Reseting question status");
          query = null;
          return result;
        }, function (error) {
          console.error(error);
          return error;
        });
      },
      lockQuestions: function (qId) {
        return updateTable('ExamType', 'Enabled', 0, 'Id>' + qId).then(function (result) { //lock exams
          console.log("Block unpaid questions");
          query = null;
          return result;
        }, function (error) {
          console.error(error);
          return error;
        });
      },

      setFreeVersion: function (examId) {
        query = "UPDATE ExamType SET Enabled=0 WHERE Id > ?";
        return $cordovaSQLite.execute(self.db, query, [examId]).then(function (result) {
          console.log("Examns locked for free version");
          query = null;
        }, function (error) {
          console.error(error);
        });
      },

      setFullVersion: function () {
        query = "UPDATE ExamType SET Enabled=1";
        return $cordovaSQLite.execute(self.db, query, []).then(function (result) {
          console.log("Exams unlocked for full version");
          query = null;
        }, function (error) {
          console.error(error);
        });
      },

      preloadTest: function (examId) {
        query = "SELECT CatId, Qty, MinScore, Duration, ExamTitle, Progress, NextExam FROM ExamType WHERE Id=? AND Enabled=1";
        return $cordovaSQLite.execute(self.db, query, [examId]).then(function (result) {
          console.log("Preloading test information");
          query = null;
          return result;
        }, function (error) {
          console.error(error);
          return error;
        });
      },
      getPropertyValue: function (propertyName) {
        query = "SELECT value FROM Config WHERE property=?";
        return $cordovaSQLite.execute(self.db, query, [propertyName]).then(function (result) {
          console.log("getting property: " + propertyName + " value: " + result.rows.item(0).value);
          query = null;
          return result.rows.item(0).value;
        }, function (error) {
          console.error(error);
          return error;
        });
      },
      getAppVersion: function () {
        return this.getPropertyValue('appVersion').then(function (result) {
          console.log(result);
          return result;
        }, function (error) {
          console.error(error);
          return error;
        });
      },
      getDBVersion: function () {
        return this.getPropertyValue('dbVersion').then(function (result) {
          console.log(result);
          return result;
        }, function (error) {
          console.error(error);
          return error;
        });
      },
      getIds: function (cat) {
        var QuestionIDs = [];
        query = "SELECT Id FROM Question WHERE CatId=? AND Enabled=1 AND Status<2";
        return $cordovaSQLite.execute(self.db, query, [cat]).then(function (result) {
          //console.log("Quantity: " + result.rows.length);
          for (j = 0; j < result.rows.length; j++) {
            QuestionIDs.push(result.rows.item(j).Id);
          }
          //console.log(QuestionIDs, QuestionIDs.length);
          return QuestionIDs;
        }, function (e) {
          console.error(e);
          return e;
        });
      },
      unlockExamById: function (examId) {
        query = "UPDATE ExamType SET Enabled=1 WHERE Id=?";
        return $cordovaSQLite.execute(self.db, query, [examId]).then(function (result) {
          console.log("ExamId: " + examId + " now Enabled");
          query = null;
        }, function (error) {
          console.error(error);
        });
      },


      getEnabledExams: function () {
        var data = [];
        query = "SELECT * from ExamType WHERE Enabled=1";
        return $cordovaSQLite.execute(self.db, query, []).then(function (result) {
          if (result.rows.length > 0) {
            for (j = 0; j < result.rows.length; j++) {
              var examData = {};
              examData.Id = result.rows.item(j).Id;
              examData.ExamTitle = result.rows.item(j).ExamTitle;
              examData.Icon = result.rows.item(j).Icon;
              data.push(examData);
            }
          }
          return data;
        }, function (e) {
          console.error(e);
          return e;
        });
      },
      getQuestionCountEnabledInCat: function (cat) {
        query = "SELECT count(*) AS qty from Question WHERE CatId=? AND Enabled=1";
        return $cordovaSQLite.execute(self.db, query, [cat]).then(function (result) {
          //console.log(result.rows.item(0).qty);
          return result.rows.item(0).qty;
        }, function (e) {
          console.error(e);
          return e;
        });
      },
      getQuestionCountByCat: function () {
        var ArrayQ = [];
        query = "SELECT DISTINCT c.Category, (SELECT COUNT(*) FROM Question WHERE CatId=c.CatId) AS qty FROM Category AS c, Question AS q WHERE c.CatId=q.CatId ORDER BY Category";
        return $cordovaSQLite.execute(self.db, query, []).then(function (result) {
          for (j = 0; j < result.rows.length; j++) {
            var List = {};
            //console.log(result.rows.item(j));
            List.catname = result.rows.item(j).Category;
            List.qty = result.rows.item(j).qty;
            ArrayQ.push(List);
          }
          //console.log(ArrayQ);
          return ArrayQ;
        }, function (e) {
          console.error(e);
          return e;
        });
      },
      getQuestionsByCat: function (cat, qty) {
        var ArrayQ = [];
        query = "SELECT * FROM Question WHERE Question.CatId=" + cat + " AND Question.Enabled=1 AND Question.Status<2 ORDER BY RANDOM() LIMIT " + qty;
        return $cordovaSQLite.execute(self.db, query, []).then(function (result) {
          //console.log(query, result);
          if (result.rows.length === qty) {
            for (i = 0; i < result.rows.length; i++) {
              ArrayQ.push(result.rows.item(i));
            }
            //console.log(JSON.stringify(ArrayQ));
            return ArrayQ;
          } else {
            return -1;
          }
        }, function (e) {
          console.error(e);
          return e;
        });
      },
      getProgressesByUserId: function (user) {
        var ArrayQ = [];
        //query = "SELECT DISTINCT ExamId, ExamTitle FROM Progress as p, ExamType as e WHERE p.ExamId=e.Id AND p.UserName=? ORDER BY p.ExamId";
        query = "SELECT DISTINCT ExamId, ExamTitle FROM Progress WHERE UserName=? ORDER BY ExamId";
        return $cordovaSQLite.execute(self.dbp, query, [user]).then(function (result) {
          for (j = 0; j < result.rows.length; j++) {
            var List = {};
            //console.log(result.rows.item(j));
            List.title = result.rows.item(j).ExamTitle;
            List.id = result.rows.item(j).ExamId;
            ArrayQ.push(List);
          }
          //console.log(ArrayQ);
          return ArrayQ;
        }, function (e) {
          console.error(e);
          return e;
        });
      },
      updateProgress: function (user, examId, examTitle, testScore, spentTime) {
        //query was updated as progress is now residing in a different database @2016-06-15
        query = "INSERT INTO Progress VALUES(?,?,?,(SELECT strftime('%s','now')),?,?);";
        return $cordovaSQLite.execute(self.dbp, query, [user, examId, examTitle, testScore, spentTime]).then(function (result) {
          console.log("INSERTED Score:" + testScore + " for User: " + user + " in Test: " + examId + " in " + spentTime + "s");
          query = null;
        }, function (error) {
          console.error(error);
        });
      },
      loadProgress: function (user, lsr, examid) {
        if (examid) {
          query = "SELECT datetime(Progress.Date,'unixepoch') as dt_time, Scored, ExamId, Time, CASE WHEN (Scored<80) THEN 'NO' ELSE 'YES' END AS Passed FROM Progress where UserName=? and ExamId=" + examid + " ORDER BY Date DESC limit ?";
        } else {
          query = "SELECT datetime(Progress.Date,'unixepoch') as dt_time, Scored, ExamId, Time, CASE WHEN (Scored<80) THEN 'NO' ELSE 'YES' END AS Passed FROM Progress where UserName=? ORDER BY Date DESC limit ?";
        }
        return $cordovaSQLite.execute(self.dbp, query, [user, lsr]).then(function (result) {
          return result;
        }, function (e) {
          console.error(e);
          return e;
        });
      },
      getQuestionsById: function (qty, ids) {
        var randomNum, qId;
        var ArrayQ = [];

        function getQuestionById(id) {
          query = "SELECT * FROM Question WHERE Id = ?";
          return $cordovaSQLite.execute(self.db, query, [id]).then(function (result) {
            return result.rows.item(0);
          }, function (e) {
            console.error(e);
            return e;
          });
        }

        for (i = 0; i < qty; i++) {
          randomNum = Math.floor(Math.random() * ids.length);
          qId = ids[randomNum];
          //ArrayQ.push(getQuestionById(qId));
          getQuestionById(qId).then(function (res) {
            ArrayQ.push(res);
          });
        }
        //console.log(ArrayQ);
        return ArrayQ;
      }
    };
  }]);
